package com.wechi.adweb.bridge.google.gtm.service;

import com.google.api.client.googleapis.javanet.GoogleNetHttpTransport;
import com.google.api.client.json.gson.GsonFactory;
import com.google.api.client.util.Lists;
import com.google.api.services.tagmanager.TagManager;
import com.google.api.services.tagmanager.TagManagerScopes;
import com.google.api.services.tagmanager.model.Account;
import com.google.api.services.tagmanager.model.Container;
import com.google.auth.http.HttpCredentialsAdapter;
import com.google.auth.oauth2.GoogleCredentials;
import com.wechi.adweb.bridge.exception.DataException;
import com.wechi.adweb.bridge.google.gtm.dto.GTMAccountDTO;
import com.wechi.adweb.bridge.google.gtm.dto.GTMContainerDTO;
import com.wechi.adweb.bridge.util.JsonUtils;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.List;

import javax.annotation.PostConstruct;

/**
 * @author wfansh
 */
@Slf4j
@Service
public class GTMService {

    @Value("${google.service.account.key}")
    private String serviceAccountKey;

    private TagManager tagManager;

    @PostConstruct
    private void init() throws GeneralSecurityException, IOException {
        // Service account authorization;
        GoogleCredentials credentials =
                GoogleCredentials.fromStream(
                                this.getClass()
                                        .getClassLoader()
                                        .getResourceAsStream(serviceAccountKey))
                        .createScoped(TagManagerScopes.all());

        this.tagManager =
                new TagManager.Builder(
                                GoogleNetHttpTransport.newTrustedTransport(),
                                GsonFactory.getDefaultInstance(),
                                new HttpCredentialsAdapter(credentials))
                        .build();
    }

    public List<GTMAccountDTO> listGTMAccounts() throws DataException {
        try {
            List<GTMAccountDTO> gtmAccounts = Lists.newArrayList();
            for (Account account : tagManager.accounts().list().execute().getAccount()) {
                GTMAccountDTO gtmAccount =
                        GTMAccountDTO.builder()
                                .id(account.getAccountId())
                                .resourceName(account.getPath())
                                .displayName(account.getName())
                                .supportUserPermissions(
                                        account.getFeatures().getSupportUserPermissions())
                                .supportMultipleContainers(
                                        account.getFeatures().getSupportMultipleContainers())
                                .build();

                // Adds into the list;
                gtmAccounts.add(gtmAccount);
            }

            log.info("listGTMAccounts : {}", JsonUtils.toJson(gtmAccounts));
            return gtmAccounts;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    public List<GTMContainerDTO> listGTMContainers(String accountResourceName)
            throws DataException {
        try {
            List<GTMContainerDTO> gtmContainers = Lists.newArrayList();
            for (Container container :
                    tagManager
                            .accounts()
                            .containers()
                            .list(accountResourceName)
                            .execute()
                            .getContainer()) {
                GTMContainerDTO gtmContainer =
                        GTMContainerDTO.builder()
                                .id(container.getContainerId())
                                .resourceName(container.getPath())
                                .displayName(container.getName())
                                .accountId(container.getAccountId())
                                .domainNames(container.getDomainName())
                                .publicId(container.getPublicId())
                                .tagIds(container.getTagIds())
                                .notes(container.getNotes())
                                .usageContext(container.getUsageContext())
                                .fingerprint(container.getFingerprint())
                                .build();

                // Adds into the list;
                gtmContainers.add(gtmContainer);
            }

            log.info("listGTMContainers : {}", JsonUtils.toJson(gtmContainers));
            return gtmContainers;
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new DataException(e);
        }
    }

    //    public GTMContainerDTO createGTMContainer(String accountResourceName, String displayName)
    //            throws DataException {
    //        try {
    //            Container container = new Container();
    //            container.setName(displayName);
    //            container.setUsageContext(List.of("web", "android", "ios"));
    //
    //            Container createdContainer =
    //                    tagManager
    //                            .accounts()
    //                            .containers()
    //                            .create(GTMAccountDTO.toResourceName(accountResourceName),
    // container)
    //                            .execute();
    //
    //            return null;
    //
    //        } catch (IOException e) {
    //            log.error(e.getMessage());
    //            throw new DataException(e);
    //        }
    //    }
}
